#!/bin/sh
# Copyright (C)2022..2023 @BABOLO http://www.babolo.ru/
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# $Id: param.sh,v 1.7 2023/09/09 16:44:20 babolo Exp $

dir="${1}"
if [ -z "${dir}" ]; then
   echo "Usage: ${0} DIR"
   echo "where DIR for param.make"
   exit 1
fi
mk="${dir}/param.make"

head  -27 "${0}" | tail -26 > "${mk}"

mktemp=`which mktemp`
if [ -z "${mktemp}" ]; then
   echo "mktemp not found"
   exit 1
fi

make=`make -V MAKE`
if [ -z "${make}" ]; then
   make=make
fi
make=`which "${make}"`
if [ -z "${make}" ]; then
   echo "make not found"
   exit 1
fi

echo=`"${make}" -V ECHO`
if [ -z "${echo}" ]; then
   echo=echo
fi

cc=`"${make}" -V CC`
if [ -z "${cc}" ]; then
   cc=cc
fi
cc=`which "${cc}"`
if [ -z "${cc}" ]; then
   echo "cc not found"
   exit 1
fi

mv=`"${make}" -V MV`
if [ -z "${mv}" ]; then
   mv=mv
fi
mv=`which "${mv}"`
if [ -z "${mv}" ]; then
   echo "mv not found"
   exit 1
fi

rm=`"${make}" -V RM`
if [ -z "${rm}" ]; then
   rm=rm
fi
rm=`which "${rm}"`
if [ -z "${rm}" ]; then
   echo "rm not found"
   exit 1
fi

sed=`"${make}" -V SED`
if [ -z "${sed}" ]; then
   sed=sed
fi
sed=`which "${sed}"`
if [ -z "${sed}" ]; then
   echo "sed not found"
   exit 1
fi

find=`"${make}" -V FIND`
if [ -z "${find}" ]; then
   find=find
fi
find=`which "${find}"`
if [ -z "${find}" ]; then
   echo "find not found"
   exit 1
fi

head=`"${make}" -V HEAD`
if [ -z "${head}" ]; then
   head=head
fi
head=`which "${head}"`
if [ -z "${head}" ]; then
   echo "head not found"
   exit 1
fi

sort=`"${make}" -V SORT`
if [ -z "${sort}" ]; then
   sort=sort
fi
sort=`which "${sort}"`
if [ -z "${sort}" ]; then
   echo "sort not found"
   exit 1
fi

f=cctest
${rm} -f "${f}" "${f}.c"
"${echo}" 'int t[] = {-909194803, -185342476};' > "${f}.c"
if ! "${cc}" -pipe -nostdlib -Wl,--oformat=binary,--entry=t -o "${f}" "${f}.c" > /dev/null; then
   NEEDGCC=1
elif ! "${echo}" -n '' | cmp -s /dev/stdin "${f}"; then
   NEEDGCC=1
fi

if [ -n "${NEEDGCC}" ]; then
   path=`"${echo}" ${PATH} | "${sed}" -e 's|:| |g'`
   CCLEXOR=` for i in ${path};                        \
             do "${find}" $i -name gcc\* || true;     \
             done                                     \
           | "${sed}" -Ene 's|^/.+/(gcc[0-9]*)$|\1|p' \
           | "${sort}" -u                             \
           `
fi

if [ -n "${NEEDGCC}" ]; then
   "${echo}" "NEEDGCC           ?= ${NEEDGCC}" >> "${mk}"
fi

j=""
for i in ${CCLEXOR}; do
    if "${i}" -pipe -nostdlib -Wl,--oformat=binary,--entry=t -o "${f}" "${f}.c" > /dev/null; then
       if "${echo}" -n '' | cmp -s /dev/stdin "${f}"; then
          j="${j} ${i}"
    fi;fi
done

for i in ${j}; do
    "${echo}" "CCLEXOR           ?= ${i}" >> "${mk}"
done

${rm} -f "${f}" "${f}.c"
